@@ -1,6 +1,6 @@
 from .miner import Miner
 from .plugin import Plugin
 
-__version__ = '0.6.10'
+__version__ = "0.6.10"
 
-__all__ = ['Miner', 'Plugin']
+__all__ = ["Miner", "Plugin"]
