@@ -67,17 +67,39 @@ class Miner(object):
     """
 
     def __init__(
-            self, alchemistic_directory, model, optimizer, loss_func,
-            code="geass", train_dataloader=None, val_dataloader=None,
-            resume=True, eval_stride=1, persist_stride=1, gpu=True,
-            drawer='matplotlib', hooks={}, max_epochs=None, statable={},
-            logging_format=None, trival=False, in_notebook=False, plugins=[],
-            logger=None, sheet=None, accumulated_iter=1, ignore_optimizer_resume=False,
-            forward=None, verbose=False, amp=False, amp_scaler=True):
+        self,
+        alchemistic_directory,
+        model,
+        optimizer,
+        loss_func,
+        code="geass",
+        train_dataloader=None,
+        val_dataloader=None,
+        resume=True,
+        eval_stride=1,
+        persist_stride=1,
+        gpu=True,
+        drawer="matplotlib",
+        hooks={},
+        max_epochs=None,
+        statable={},
+        logging_format=None,
+        trival=False,
+        in_notebook=False,
+        plugins=[],
+        logger=None,
+        sheet=None,
+        accumulated_iter=1,
+        ignore_optimizer_resume=False,
+        forward=None,
+        verbose=False,
+        amp=False,
+        amp_scaler=True,
+    ):
         self.alchemistic_directory = alchemistic_directory
         self.code = code
         if trival:
-            self.code = f'trival_{code}'
+            self.code = f"trival_{code}"
         self.create_dirs()
         self.gpu = gpu
         self.devices = torch.device("cuda" if torch.cuda.is_available() else "cpu")
@@ -87,7 +109,7 @@ class Miner(object):
             self.set_logging_config(alchemistic_directory, self.code, logging_format)
             self.logger = logging
         self.create_drawer(drawer)
-        self.models_dir = os.path.join(alchemistic_directory, self.code, 'models')
+        self.models_dir = os.path.join(alchemistic_directory, self.code, "models")
         self.in_notebook = in_notebook
         self.statable = statable
         self.accumulated_iter = float(accumulated_iter)
@@ -103,8 +125,8 @@ class Miner(object):
         self.resume = resume
         self.eval_stride = eval_stride
         self.persist_stride = persist_stride
-        self.lowest_train_loss = float('inf')
-        self.lowest_val_loss = float('inf')
+        self.lowest_train_loss = float("inf")
+        self.lowest_val_loss = float("inf")
         self.current_epoch = 0
         self.current_train_iteration = 0
         self.current_val_iteration = 0
@@ -128,25 +150,25 @@ class Miner(object):
             plugin.set_miner(self)
 
         self._set_tqdm()
-        self.call_hook_func('before_init')
+        self.call_hook_func("before_init")
         self._check_statable()
         self.init_model()
         if self.sheet:
             self.sheet_progress = dict(
-                epoch=0,
-                train_percentage='0%',
-                val_percentage='0%'
+                epoch=0, train_percentage="0%", val_percentage="0%"
             )
             self.last_flushed_at = 0
             self.sheet.onready()
             self.sheet.flush()
-        self.status = 'init'
-        self.call_hook_func('after_init')
+        self.status = "init"
+        self.call_hook_func("after_init")
 
     def _check_statable(self):
         for name, statable in self.statable.items():
-            if not (hasattr(statable, 'state_dict') and hasattr(statable, 'load_state_dict')):
-                raise Exception(f'The {name} is not a statable object')
+            if not (
+                hasattr(statable, "state_dict") and hasattr(statable, "load_state_dict")
+            ):
+                raise Exception(f"The {name} is not a statable object")
 
     def _set_tqdm(self):
         if self.in_notebook:
@@ -157,10 +179,10 @@ class Miner(object):
     def _init_sheet(self):
         self.sheet.set_miner(self)
         self.sheet.reset_index()
-        self.sheet.create_column('code', 'Code')
-        self.sheet.create_column('progress', 'Progress')
-        self.sheet.create_column('loss', 'Loss')
-        self.sheet.update('code', self.code)
+        self.sheet.create_column("code", "Code")
+        self.sheet.create_column("progress", "Progress")
+        self.sheet.create_column("loss", "Loss")
+        self.sheet.update("code", self.code)
 
     def create_sheet_column(self, key, title):
         if self.sheet is None:
@@ -174,66 +196,70 @@ class Miner(object):
 
     def set_logging_config(self, alchemistic_directory, code, logging_format):
         self.log_dir = os.path.join(alchemistic_directory, code)
-        log_file = os.path.join(self.log_dir, 'log.txt')
-        logging_format = logging_format if logging_format is not None else \
-            '%(levelname)s %(asctime)s %(message)s'
+        log_file = os.path.join(self.log_dir, "log.txt")
+        logging_format = (
+            logging_format
+            if logging_format is not None
+            else "%(levelname)s %(asctime)s %(message)s"
+        )
         logging.basicConfig(
             filename=log_file,
             format=logging_format,
             datefmt="%m-%d %H:%M:%S",
-            level=logging.INFO
+            level=logging.INFO,
         )
 
     def create_drawer(self, drawer):
-        if drawer == 'tensorboard':
+        if drawer == "tensorboard":
             self.drawer = drawers.TensorboardDrawer(self)
-        elif drawer == 'matplotlib':
+        elif drawer == "matplotlib":
             self.drawer = drawers.MatplotlibDrawer(self)
         else:
             self.drawer = drawer
 
-    def notebook_output(self, message, _type='info'):
+    def notebook_output(self, message, _type="info"):
         type_config = {
-            'info': ['💬', '#6f818a'],
-            'success': ['✅', '#7cb305'],
-            'error': ['❌', '#cf1322'],
-            'warning': ['⚠️', '#d46b08'],
+            "info": ["💬", "#6f818a"],
+            "success": ["✅", "#7cb305"],
+            "error": ["❌", "#cf1322"],
+            "warning": ["⚠️", "#d46b08"],
         }[_type]
         if self.in_notebook:
-            display(HTML(
-                f'<div style="font-size: 12px; color: {type_config[1]}">'
-                f'⏰ {time.strftime("%b %d - %H:%M:%S")} >>> '
-                f'{type_config[0]} {message}'
-                '</div>'
-            ))
+            display(
+                HTML(
+                    f'<div style="font-size: 12px; color: {type_config[1]}">'
+                    f'⏰ {time.strftime("%b %d - %H:%M:%S")} >>> '
+                    f"{type_config[0]} {message}"
+                    "</div>"
+                )
+            )
 
     def notebook_divide(self, message):
         if self.in_notebook:
-            display(HTML(
-                '<div style="display: flex; justify-content: center;">'
-                f'<h3 style="color: #7cb305; border-bottom: 4px dashed #91d5ff; padding-bottom: 6px;">{message}</h3>'
-                '</div>'
-            ))
+            display(
+                HTML(
+                    '<div style="display: flex; justify-content: center;">'
+                    f'<h3 style="color: #7cb305; border-bottom: 4px dashed #91d5ff; padding-bottom: 6px;">{message}</h3>'
+                    "</div>"
+                )
+            )
 
     def init_model(self):
-        """resume from some checkpoint
-        """
+        """resume from some checkpoint"""
         if isinstance(self.model, torch.nn.DataParallel):
             raise Exception(
-                'Don\'t parallel the model yourself, instead, if the '
-                '`gpu` option is true(default), Minetorch will do this for you.'
+                "Don't parallel the model yourself, instead, if the "
+                "`gpu` option is true(default), Minetorch will do this for you."
             )
 
         if self.resume is True:
             # resume from the newest model
-            if self.model_file_path('latest') is not None:
-                checkpoint_path = self.model_file_path('latest')
+            if self.model_file_path("latest") is not None:
+                checkpoint_path = self.model_file_path("latest")
             else:
                 checkpoint_path = None
-                msg = ('Could not find checkpoint to resume, '
-                       'train from scratch')
-                self.logger.warning(msg)
-                self.notebook_output(msg, _type='warning')
+                msg = "Could not find checkpoint to resume, " "train from scratch"
+                self.notify(msg, "warning")
         elif isinstance(self.resume, str):
             checkpoint_path = self.model_file_path(self.resume)
         elif isinstance(self.resume, int):
@@ -247,74 +273,76 @@ class Miner(object):
 
         if checkpoint_path is not None:
             msg = f"Start to load checkpoint {checkpoint_path}"
-            self.logger.info(msg)
-            self.notebook_output(msg)
+            self.notify(msg)
             checkpoint = torch.load(checkpoint_path)
-            self.current_epoch = checkpoint.get('epoch', 0)
-            self.current_train_iteration = checkpoint.get('train_iteration', 0)
-            self.current_val_iteration = checkpoint.get('val_iteration', 0)
-            self.lowest_train_loss = checkpoint.get('lowest_train_loss', 9999)
-            self.lowest_val_loss = checkpoint.get('lowest_val_loss', 9999)
+            self.current_epoch = checkpoint.get("epoch", 0)
+            self.current_train_iteration = checkpoint.get("train_iteration", 0)
+            self.current_val_iteration = checkpoint.get("val_iteration", 0)
+            self.lowest_train_loss = checkpoint.get("lowest_train_loss", 9999)
+            self.lowest_val_loss = checkpoint.get("lowest_val_loss", 9999)
 
             # load model state
             try:
-                self.model.load_state_dict(checkpoint['state_dict'], strict=True)
+                self.model.load_state_dict(checkpoint["state_dict"], strict=True)
             except Exception as e:
-                msg = (f'load checkpoint failed with {e}, the state in the '
-                       'checkpoint is not matched with the model, '
-                       'try to reload checkpoint with unstrict mode')
-                self.logger.warning(msg)
-                self.notebook_output(msg)
-                self.model.load_state_dict(checkpoint['state_dict'], strict=False)
+                msg = (
+                    f"load checkpoint failed with {e}, the state in the "
+                    "checkpoint is not matched with the model, "
+                    "try to reload checkpoint with unstrict mode"
+                )
+                self.notify(msg, "warning")
+                self.model.load_state_dict(checkpoint["state_dict"], strict=False)
 
             # load optimizer state
-            if 'optimizer' in checkpoint and not self.ignore_optimizer_resume:
+            if "optimizer" in checkpoint and not self.ignore_optimizer_resume:
                 try:
-                    self.optimizer.load_state_dict(checkpoint['optimizer'])
+                    self.optimizer.load_state_dict(checkpoint["optimizer"])
                 except Exception as e:
-                    msg = (f'load optimizer state failed with {e}, will skip this error and continue, '
-                           'stop the process if it is not expected')
-                    self.logger.warning(msg)
-                    self.notebook_output(msg)
+                    msg = (
+                        f"load optimizer state failed with {e}, will skip this error and continue, "
+                        "stop the process if it is not expected"
+                    )
+                    self.notify(msg, "warning")
 
             # load drawer state
-            if (self.drawer is not None) and ('drawer_state' in checkpoint):
-                self.drawer.set_state(checkpoint['drawer_state'])
+            if (self.drawer is not None) and ("drawer_state" in checkpoint):
+                self.drawer.set_state(checkpoint["drawer_state"])
 
             # load scaler state
             if self.amp and self.amp_scaler:
                 try:
-                    self.optimizer.load_state_dict(checkpoint['scaler'])
+                    self.scaler.load_state_dict(checkpoint["scaler"])
                 except Exception as e:
-                    msg = (f'load scaler state failed with {e}, will skip this error and continue, '
-                            'stop the process if it is not expected')
-                    self.logger.warning(msg)
-                    self.notebook_output(msg)
+                    msg = (
+                        f"load scaler state failed with {e}, will skip this error and continue, "
+                        "stop the process if it is not expected"
+                    )
+                    self.notify(msg, "warning")
 
             # load other statable state
-            if 'statable' in checkpoint:
+            if "statable" in checkpoint:
                 for name, statable in self.statable.items():
-                    if name not in checkpoint['statable']:
+                    if name not in checkpoint["statable"]:
                         continue
-                    statable.load_state_dict(checkpoint['statable'][name])
-            msg = 'checkpoint loaded'
-            self.notebook_output(msg, _type='success')
+                    statable.load_state_dict(checkpoint["statable"][name])
+            msg = "checkpoint loaded"
+            self.notify(msg, "success")
         self.model = self.parallel_model(self.model)
 
     def parallel_model(self, model):
         if self.gpu:
             gpu_count = torch.cuda.device_count()
             if gpu_count == 0:
-                self.notify('no GPU detected, will train on CPU.')
+                self.notify("no GPU detected, will train on CPU.")
             else:
-                self.notify(f'found {gpu_count} GPUs, will use all of them to train')
-                devices = list(map(lambda x: f'cuda:{x}', range(gpu_count)))
+                self.notify(f"found {gpu_count} GPUs, will use all of them to train")
+                devices = list(map(lambda x: f"cuda:{x}", range(gpu_count)))
                 model.cuda()
                 model = torch.nn.DataParallel(model, devices)
         return model
 
-    def notify(self, message, _type='info'):
-        getattr(self.logger, _type)(message)
+    def notify(self, message, _type="info"):
+        getattr(self.logger, "info" if _type == "success" else _type)(message)
         self.notebook_output(message, _type)
 
     def call_hook_func(self, name, **payload):
@@ -328,20 +356,24 @@ class Miner(object):
                 getattr(plugin, name)(**payload)
 
     def train(self):
-        """start to train the model
-        """
+        """start to train the model"""
         while True:
             self.current_epoch += 1
-            self.call_hook_func('before_epoch_start', epoch=self.current_epoch)
-            self.notebook_divide(f'Epoch {self.current_epoch}')
+            self.call_hook_func("before_epoch_start", epoch=self.current_epoch)
+            self.notebook_divide(f"Epoch {self.current_epoch}")
             self.model.train()
             train_iters = len(self.train_dataloader)
 
             total_train_loss = 0
             percentage = 0
             total = len(self.train_dataloader)
-            self.notebook_output(f'start to train epoch {self.current_epoch}')
-            self._update_progress(force=True, epoch=self.current_epoch, train_percentage='0%', val_percentage='0%')
+            self.notify(f"start to train epoch {self.current_epoch}")
+            self._update_progress(
+                force=True,
+                epoch=self.current_epoch,
+                train_percentage="0%",
+                val_percentage="0%",
+            )
             t = self.tqdm(self.train_dataloader)
             for index, data in enumerate(t):
                 if self.trival is True and index == 10:
@@ -358,15 +390,17 @@ class Miner(object):
                 total_train_loss += train_loss
                 current_percentage = math.ceil(index / total * 100)
                 if current_percentage != percentage:
-                    self._update_progress(train_percentage=f'{percentage}%')
+                    self._update_progress(train_percentage=f"{percentage}%")
                     percentage = current_percentage
             self.optimizer.step()
             self.optimizer.zero_grad(set_to_none=True)
-            self._update_progress(force=True, train_percentage=f'{current_percentage}%')
+            self._update_progress(force=True, train_percentage=f"{current_percentage}%")
 
             total_train_loss = total_train_loss / train_iters
-            self.notebook_output(f'training of epoch {self.current_epoch} finished, '
-                                 f'loss is {total_train_loss}')
+            self.notify(
+                f"training of epoch {self.current_epoch} finished, "
+                f"loss is {total_train_loss}"
+            )
 
             total_val_loss = 0
             percentage = 0
@@ -375,7 +409,7 @@ class Miner(object):
                 val_iters = len(self.val_dataloader)
                 with torch.set_grad_enabled(False):
                     self.model.eval()
-                    self.notebook_output(f'validate epoch {self.current_epoch}')
+                    self.notify(f"validate epoch {self.current_epoch}")
                     t = self.tqdm(self.val_dataloader)
                     for index, data in enumerate(t):
                         if self.trival is True and index == 10:
@@ -385,60 +419,71 @@ class Miner(object):
                         total_val_loss += val_loss
                         current_percentage = math.ceil(index / total * 100)
                         if current_percentage != percentage:
-                            self._update_progress(val_percentage=f'{percentage}%')
+                            self._update_progress(val_percentage=f"{percentage}%")
                             percentage = current_percentage
-                    self._update_progress(force=True, val_percentage=f'{current_percentage}%')
+                    self._update_progress(
+                        force=True, val_percentage=f"{current_percentage}%"
+                    )
 
                 total_val_loss = total_val_loss / val_iters
-                self.notebook_output(f'validation of epoch {self.current_epoch}'
-                                     f'finished, loss is {total_val_loss}')
+                self.notify(
+                    f"validation of epoch {self.current_epoch}"
+                    f"finished, loss is {total_val_loss}"
+                )
             if self.drawer is not None:
                 png_file = self.drawer.scalars(
-                    self.current_epoch, {'train': total_train_loss, 'val': total_val_loss}, 'loss'
+                    self.current_epoch,
+                    {"train": total_train_loss, "val": total_val_loss},
+                    "loss",
                 )
                 if png_file is not None:
-                    self.update_sheet('loss', {'raw': png_file, 'processor': 'upload_image'})
+                    self.update_sheet(
+                        "loss", {"raw": png_file, "processor": "upload_image"}
+                    )
 
             if total_train_loss < self.lowest_train_loss:
                 self.lowest_train_loss = total_train_loss
 
             if total_val_loss < self.lowest_val_loss:
-                message = ('current val loss {} is lower than lowest {}, '
-                           'persist this model as best one'.format(
-                            total_val_loss, self.lowest_val_loss))
-                self.notebook_output(f'{message}', _type='success')
-                self.logger.info(message)
+                message = (
+                    "current val loss {} is lower than lowest {}, "
+                    "persist this model as best one".format(
+                        total_val_loss, self.lowest_val_loss
+                    )
+                )
+                self.notify(message, "success")
 
                 self.lowest_val_loss = total_val_loss
-                self.persist('best')
-            self.persist('latest')
+                self.persist("best")
+            self.persist("latest")
 
             if not self.current_epoch % self.persist_stride:
-                self.persist('epoch_{}'.format(self.current_epoch))
+                self.persist("epoch_{}".format(self.current_epoch))
 
             if self.max_epochs is not None and self.current_epoch >= self.max_epochs:
-                self.call_hook_func('before_quit')
-                self.logger.info('exceed max epochs, quit!')
+                self.call_hook_func("before_quit")
+                self.notify("exceed max epochs, quit!")
                 break
 
             if self.sheet:
                 self.sheet.flush()
             self.call_hook_func(
-                'after_epoch_end',
+                "after_epoch_end",
                 train_loss=total_train_loss,
                 val_loss=total_val_loss,
-                epoch=self.current_epoch
+                epoch=self.current_epoch,
             )
 
     def run_train_iteration(self, index, data, train_iters):
-        self.status = 'train'
+        self.status = "train"
         self.current_train_iteration += 1
         self.call_hook_func(
-            'before_train_iteration_start',
+            "before_train_iteration_start",
             data=data,
             index=index,
             total_iters=train_iters,
-            iteration=self.current_train_iteration)
+            iteration=self.current_train_iteration,
+        )
         if self.amp:
             with torch.cuda.amp.autocast():
                 _, loss = self._forward(data)
@@ -451,16 +496,19 @@ class Miner(object):
         seperate_loss.backward()
         loss = loss.detach().cpu().item()
         if self.verbose:
-            self.logger.info('[train {}/{}/{}] loss {}'.format(
-                self.current_epoch, index, train_iters, loss))
+            self.logger.info(
+                "[train {}/{}/{}] loss {}".format(
+                    self.current_epoch, index, train_iters, loss
+                )
+            )
 
         self.call_hook_func(
-            'after_train_iteration_end',
+            "after_train_iteration_end",
             loss=loss,
             data=data,
             index=index,
             total_iters=train_iters,
-            iteration=self.current_train_iteration
+            iteration=self.current_train_iteration,
         )
         return loss
 
@@ -473,35 +521,37 @@ class Miner(object):
             return predict, loss
 
     def run_val_iteration(self, index, data, val_iters):
-        self.status = 'val'
+        self.status = "val"
         self.current_val_iteration += 1
         self.call_hook_func(
-            'before_val_iteration_start',
+            "before_val_iteration_start",
             data=data,
             index=index,
             total_iters=val_iters,
-            iteration=self.current_val_iteration
+            iteration=self.current_val_iteration,
         )
         predict, loss = self._forward(data)
         loss = loss.detach().cpu().item()
         if self.verbose:
-            self.logger.info('[val {}/{}/{}] loss {}'.format(
-                self.current_epoch, index, val_iters, loss))
+            self.logger.info(
+                "[val {}/{}/{}] loss {}".format(
+                    self.current_epoch, index, val_iters, loss
+                )
+            )
         self.call_hook_func(
-            'after_val_iteration_ended',
+            "after_val_iteration_ended",
             predicts=predict,
             loss=loss,
             data=data,
             index=index,
             total_iters=val_iters,
-            iteration=self.current_val_iteration
+            iteration=self.current_val_iteration,
         )
         return loss
 
     def persist(self, name):
-        """save the model to disk
-        """
-        self.call_hook_func('before_checkpoint_persisted')
+        """save the model to disk"""
+        self.call_hook_func("before_checkpoint_persisted")
         if self.drawer is not None:
             drawer_state = self.drawer.get_state()
         else:
@@ -513,32 +563,31 @@ class Miner(object):
             model_state_dict = self.model.state_dict()
 
         state = {
-            'state_dict': model_state_dict,
-            'optimizer': self.optimizer.state_dict(),
-            'epoch': self.current_epoch,
-            'train_iteration': self.current_train_iteration,
-            'val_iteration': self.current_val_iteration,
-            'lowest_train_loss': self.lowest_train_loss,
-            'lowest_val_loss': self.lowest_val_loss,
-            'drawer_state': drawer_state,
-            'statable': {}
+            "state_dict": model_state_dict,
+            "optimizer": self.optimizer.state_dict(),
+            "epoch": self.current_epoch,
+            "train_iteration": self.current_train_iteration,
+            "val_iteration": self.current_val_iteration,
+            "lowest_train_loss": self.lowest_train_loss,
+            "lowest_val_loss": self.lowest_val_loss,
+            "drawer_state": drawer_state,
+            "statable": {},
         }
 
         for statable_name, statable in self.statable.items():
-            state['statable'][statable_name] = statable.state_dict()
+            state["statable"][statable_name] = statable.state_dict()
 
         if self.amp and self.amp_scaler:
-            state['scaler'] = self.scaler.state_dict()
+            state["scaler"] = self.scaler.state_dict()
 
         modelpath = self.standard_model_path(name)
         torch.save(state, modelpath)
-        message = f'save checkpoint to {self.standard_model_path(name)}'
-        self.logger.info(message)
-        self.notebook_output(message)
-        self.call_hook_func('after_checkpoint_persisted', modelpath=modelpath)
+        message = f"save checkpoint to {self.standard_model_path(name)}"
+        self.notify(message)
+        self.call_hook_func("after_checkpoint_persisted", modelpath=modelpath)
 
     def standard_model_path(self, model_name):
-        return os.path.join(self.models_dir, f'{model_name}.pth.tar')
+        return os.path.join(self.models_dir, f"{model_name}.pth.tar")
 
     def model_file_path(self, model_name):
         model_name_path = Path(str(model_name))
@@ -547,8 +596,8 @@ class Miner(object):
         search_paths = [
             model_name_path,
             models_dir_path / model_name_path,
-            models_dir_path / f'{model_name}.pth.tar',
-            models_dir_path / f'epoch_{model_name}.pth.tar',
+            models_dir_path / f"{model_name}.pth.tar",
+            models_dir_path / f"epoch_{model_name}.pth.tar",
         ]
 
         for path in search_paths:
@@ -559,25 +608,21 @@ class Miner(object):
 
     # TODO: implement methods below
     def graceful_stop(self):
-        """stop train and exist after this epoch
-        """
+        """stop train and exist after this epoch"""
         pass
 
     def save_and_stop(self):
-        """save the model immediately and stop training
-        """
+        """save the model immediately and stop training"""
         pass
 
     def create_dirs(self):
-        """Create directories
-        """
-        self.create_dir('')
+        """Create directories"""
+        self.create_dir("")
         self.create_dir(self.code)
-        self.create_dir(self.code, 'models')
+        self.create_dir(self.code, "models")
 
     def create_dir(self, *args):
-        """Create directory
-        """
+        """Create directory"""
         current_dir = self.alchemistic_directory
         for dir_name in args:
             current_dir = os.path.join(current_dir, dir_name)
@@ -604,5 +649,5 @@ class Miner(object):
 train progress:  {self.sheet_progress.get('train_percentage')}
   val progress:  {self.sheet_progress.get('val_percentage')}
 """
-        self.sheet.update('progress', progress)
+        self.sheet.update("progress", progress)
         self.periodly_flush(force)
